@@ -246,6 +246,73 @@ fn basic() {
     p.cargo("test").build_std(&setup).target_host().run();
 }
 
+#[cargo_test(build_std_mock)]
+fn shared_std_dependency_rebuild() {
+    let manifest_dir = std::env::var("CARGO_MANIFEST_DIR").unwrap();
+    let setup = setup();
+    let p = project()
+        .file(
+            "Cargo.toml",
+            format!(
+                "
+                [package]
+                name = \"foo\"
+                version = \"0.1.0\"
+                edition = \"2021\"
+
+                [build-dependencies]
+                dep_test = {{ path = \"{}/tests/testsuite/mock-std/dep_test\" }}
+            ",
+                manifest_dir
+            )
+            .as_str(),
+        )
+        .file(
+            "src/main.rs",
+            r#"
+            fn main() {
+                println!("Hello, World!");
+            }
+            "#,
+        )
+        .file(
+            "build.rs",
+            r#"
+            fn main() {
+                println!("cargo::rerun-if-changed=build.rs");
+            }
+            "#,
+        )
+        .build();
+
+    p.cargo("build -v")
+        .build_std(&setup)
+        .target_host()
+        .with_stderr_data(str![[r#"
+...
+[RUNNING] `[..] rustc --crate-name dep_test [..]`
+...
+[RUNNING] `[..] rustc --crate-name dep_test [..]`
+...
+"#]])
+        .run();
+
+    // TODO: Because of the way in which std is resolved, it's mandatory that this is left commented
+    // out as it will fail. This case should result in `dep_test` only being built once, however
+    // it's still being built twice. This is a bug.
+    //
+    //    p.cargo("build -v")
+    //        .build_std(&setup)
+    //        .with_stderr_does_not_contain(str![[r#"
+    //...
+    //[RUNNING] `[..] rustc --crate-name dep_test [..]`
+    //...
+    //[RUNNING] `[..] rustc --crate-name dep_test [..]`
+    //...
+    //"#]])
+    //        .run();
+}
+
 #[cargo_test(build_std_mock)]
 fn simple_lib_std() {
     let setup = setup();
@@ -281,7 +348,6 @@ fn simple_bin_std() {
     p.cargo("run -v").build_std(&setup).target_host().run();
 }
 
-#[expect(deprecated)]
 #[cargo_test(build_std_mock)]
 fn lib_nostd() {
     let setup = setup();
@@ -323,6 +389,115 @@ fn check_core() {
         .run();
 }
 
+#[cargo_test(build_std_mock)]
+fn build_std_with_no_arg_for_core_only_target() {
+    let has_rustup_aarch64_unknown_none = std::process::Command::new("rustup")
+        .args(["target", "list", "--installed"])
+        .output()
+        .ok()
+        .map(|output| {
+            String::from_utf8(output.stdout)
+                .map(|stdout| stdout.contains("aarch64-unknown-none"))
+                .unwrap_or_default()
+        })
+        .unwrap_or_default();
+    if !has_rustup_aarch64_unknown_none {
+        let msg =
+            "to run this test, run `rustup target add aarch64-unknown-none --toolchain nightly`";
+        if cargo_util::is_ci() {
+            panic!("{msg}");
+        } else {
+            eprintln!("{msg}");
+        }
+        return;
+    }
+
+    let setup = setup();
+
+    let p = project()
+        .file(
+            "src/lib.rs",
+            r#"
+                #![no_std]
+                pub fn foo() {
+                    assert_eq!(u8::MIN, 0);
+                }
+            "#,
+        )
+        .build();
+
+    p.cargo("build -v")
+        .arg("--target=aarch64-unknown-none")
+        .build_std(&setup)
+        .with_stderr_data(
+            str![[r#"
+[UPDATING] `dummy-registry` index
+[DOWNLOADING] crates ...
+[DOWNLOADED] registry-dep-using-std v1.0.0 (registry `dummy-registry`)
+[DOWNLOADED] registry-dep-using-core v1.0.0 (registry `dummy-registry`)
+[DOWNLOADED] registry-dep-using-alloc v1.0.0 (registry `dummy-registry`)
+[COMPILING] compiler_builtins v0.1.0 ([..]/library/compiler_builtins)
+[COMPILING] core v0.1.0 ([..]/library/core)
+[COMPILING] foo v0.0.1 ([ROOT]/foo)
+[RUNNING] `[..] rustc --crate-name compiler_builtins [..]--target aarch64-unknown-none[..]`
+[RUNNING] `[..] rustc --crate-name core [..]--target aarch64-unknown-none[..]`
+[RUNNING] `[..] rustc --crate-name foo [..]--target aarch64-unknown-none[..]`
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]]
+            .unordered(),
+        )
+        .run();
+
+    p.cargo("clean").run();
+
+    // Also work for a mix of std and core-only targets,
+    // though not sure how common it is...
+    //
+    // Note that we don't  download std dependencies for the second call
+    // because `-Zbuild-std` downloads them all also when building for core only.
+    p.cargo("build -v")
+        .arg("--target=aarch64-unknown-none")
+        .target_host()
+        .build_std(&setup)
+        .with_stderr_data(
+            str![[r#"
+[UPDATING] `dummy-registry` index
+[COMPILING] core v0.1.0 ([..]/library/core)
+[COMPILING] dep_test v0.1.0 ([..]/dep_test)
+[COMPILING] compiler_builtins v0.1.0 ([..]/library/compiler_builtins)
+[COMPILING] proc_macro v0.1.0 ([..]/library/proc_macro)
+[COMPILING] panic_unwind v0.1.0 ([..]/library/panic_unwind)
+[COMPILING] rustc-std-workspace-core v1.9.0 ([..]/library/rustc-std-workspace-core)
+[COMPILING] foo v0.0.1 ([ROOT]/foo)
+[COMPILING] registry-dep-using-core v1.0.0
+[COMPILING] alloc v0.1.0 ([..]/library/alloc)
+[COMPILING] rustc-std-workspace-alloc v1.9.0 ([..]/library/rustc-std-workspace-alloc)
+[COMPILING] registry-dep-using-alloc v1.0.0
+[COMPILING] std v0.1.0 ([..]/library/std)
+[RUNNING] `[..]rustc --crate-name compiler_builtins [..]--target aarch64-unknown-none[..]`
+[RUNNING] `[..]rustc --crate-name core [..]--target aarch64-unknown-none[..]`
+[RUNNING] `[..]rustc --crate-name foo [..]--target aarch64-unknown-none[..]`
+[RUNNING] `[..]rustc --crate-name core [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name dep_test [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name proc_macro [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name panic_unwind [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name compiler_builtins [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name rustc_std_workspace_core [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name registry_dep_using_core [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name alloc [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name rustc_std_workspace_alloc [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name registry_dep_using_alloc [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name std [..]--target [HOST_TARGET][..]`
+[RUNNING] `[..]rustc --crate-name foo [..]--target [HOST_TARGET][..]`
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+
+"#]]
+            .unordered(),
+        )
+        .run();
+}
+
 #[cargo_test(build_std_mock)]
 fn depend_same_as_std() {
     let setup = setup();
@@ -594,7 +769,6 @@ fn ignores_incremental() {
         .starts_with("foo-"));
 }
 
-#[expect(deprecated)]
 #[cargo_test(build_std_mock)]
 fn cargo_config_injects_compiler_builtins() {
     let setup = setup();
@@ -693,7 +867,6 @@ fn proc_macro_only() {
         .run();
 }
 
-#[expect(deprecated)]
 #[cargo_test(build_std_mock)]
 fn fetch() {
     let setup = setup();
