@@ -1,6 +1,6 @@
 //@ run-pass
 //@ edition: 2021
-#![feature(async_iterator, async_iter_from_iter, async_for_loop, noop_waker)]
+#![feature(async_iterator, async_iter_from_iter, async_for_loop)]
 
 use std::future::Future;
 
